@@ -69,7 +69,7 @@ class PyQuery(list):
         html = None
         elements = []
         self._base_url = None
-        parser = kwargs.get('parser')
+        self.parser = kwargs.get('parser', None)
         if 'parser' in kwargs:
             del kwargs['parser']
         if not kwargs and len(args) == 1 and isinstance(args[0], basestring) \
@@ -97,7 +97,7 @@ class PyQuery(list):
                 self._base_url = url
             else:
                 raise ValueError('Invalid keyword arguments %s' % kwargs)
-            elements = fromstring(html, parser)
+            elements = fromstring(html, self.parser)
         else:
             # get nodes
 
@@ -115,7 +115,7 @@ class PyQuery(list):
             # get context
             if isinstance(context, basestring):
                 try:
-                    elements = fromstring(context, parser)
+                    elements = fromstring(context, self.parser)
                 except Exception, e:
                     raise ValueError('%r, %s' % (e, context))
             elif isinstance(context, self.__class__):
@@ -309,6 +309,24 @@ class PyQuery(list):
         elements = [child for tag in self for child in tag.getchildren()]
         return self._filter_only(selector, elements)
 
+    def closest(self, selector=None):
+        """
+            >>> d = PyQuery('<div class="hello"><p>This is a <strong class="hello">test</strong></p></div>')
+            >>> d('strong').closest('div')
+            [<div.hello>]
+            >>> d('strong').closest('.hello')
+            [<strong.hello>]
+            >>> d('strong').closest('form')
+            []
+        """
+        try:
+            current = self[0]
+        except IndexError:
+            current = None
+        while current is not None and not self.__class__(current).is_(selector):
+            current = current.getparent()
+        return self.__class__(current, **dict(parent=self))
+
     def filter(self, selector):
         """Filter elements in self using selector (string or function).
 
@@ -671,7 +689,7 @@ class PyQuery(list):
             for tag in self:
                 for child in tag.getchildren():
                     tag.remove(child)
-                root = etree.fromstring('<root>' + new_html + '</root>')
+                root = fromstring('<root>' + new_html + '</root>', self.parser)[0]
                 children = root.getchildren()
                 if children:
                     tag.extend(children)
@@ -727,7 +745,7 @@ class PyQuery(list):
 
     def _get_root(self, value):
         if  isinstance(value, basestring):
-            root = etree.fromstring('<root>' + value + '</root>')
+            root = fromstring('<root>' + value + '</root>', self.parser)[0]
         elif isinstance(value, etree._Element):
             root = self.__class__(value)
         elif isinstance(value, PyQuery):
@@ -922,10 +940,16 @@ class PyQuery(list):
     def replaceWith(self, value):
         """replace nodes by value
         """
-        self.before(value)
-        for tag in self:
-            parent = tag.getparent()
-            parent.remove(tag)
+        if callable(value):
+            for i, element in enumerate(self):
+                self.__class__(element).before(value(i, element) + (element.tail or ''))
+                parent = element.getparent()
+                parent.remove(element)
+        else:
+            for tag in self:
+                self.__class__(tag).before(value + (tag.tail or ''))
+                parent = tag.getparent()
+                parent.remove(tag)
         return self
 
     def replaceAll(self, expr):
